@@ -1,16 +1,19 @@
+# pylint:disable=too-many-lines
+
 """
 A collection of functions for integrating with the Archimedes API
 """
-import concurrent.futures
 import json
 import os
+import sys
+from concurrent.futures import ThreadPoolExecutor
 from functools import partial
 from http import HTTPStatus
 from typing import Dict, List
 
 import pandas as pd
 import requests
-from requests.exceptions import (
+from requests.exceptions import (  # pylint:disable=redefined-builtin
     ConnectionError,
     ConnectTimeout,
     HTTPError,
@@ -19,12 +22,14 @@ from requests.exceptions import (
 )
 from retry import retry
 
-import archimedes
+import archimedes  # pylint:disable=unused-import
 from archimedes.auth import NoneAuth, get_auth
 from archimedes.configuration import get_api_base_url
 from archimedes.data.types import PredictionData
+from archimedes.utils import get_end_date, get_start_date
+from archimedes.utils.split import get_queries_observation, get_queries_observation_json
 
-DEFAULT_TIMEOUT = os.getenv("ARCHIMEDES_API_TIMEOUT", 120)  # 2 minutes
+DEFAULT_TIMEOUT = int(os.getenv("ARCHIMEDES_API_TIMEOUT", "120"))  # 2 minutes
 API_VERSION = 2
 get_api_base_url_v2 = partial(get_api_base_url, API_VERSION)
 RETRY_EXCEPTIONS = (ConnectionError, ConnectTimeout, HTTPError, Timeout)
@@ -39,7 +44,8 @@ def get_intraday_trades(
 ):
     """Get raw intraday trades from Archimedes Database
 
-    This function can be used to fetch raw time series from the Archimedes Database without any post-processing.
+    This function can be used to fetch raw time series from the Archimedes Database
+    without any post-processing.
     To see which series are available, use `list_ids()`.
 
     Example:
@@ -70,46 +76,51 @@ def get_intraday_trades(
     Raises:
         HTTPError: If an HTTP error occurs when requesting the API.
         NoneAuth: If the user is unauthorized or if the authorization has expired.
-    """
+    """  # pylint:disable=line-too-long
 
     if isinstance(price_areas, str):
         price_areas = [price_areas]
 
-    start = pd.to_datetime(start) if start else archimedes.ArchimedesConstants.DATE_LOW
-    end = pd.to_datetime(end) if end else archimedes.ArchimedesConstants.DATE_HIGH
+    start, end = get_start_date(start), get_end_date(end)
 
-    queries = [
-        {
-            "series_id": "NP/IntradayTrades",
-            "start": start,
-            "end": end,
-            "flatten_columns": False,
-        }
-    ]
-    if price_areas is not None:
-        queries = [
-            {"price_areas": price_area, **query}
-            for query in queries
-            for price_area in price_areas
-        ]
+    queries = get_queries_observation_json(
+        ["NP/IntradayTrades"], price_areas, start, end
+    )
 
     base_url = get_api_base_url_v2()
     observation_data = []
-    with concurrent.futures.ThreadPoolExecutor() as executor:
-        future_requests = [
-            executor.submit(
-                _make_api_request,
-                f"{base_url}/observation_json/get",
-                access_token=access_token,
-                params=query,
+    with ThreadPoolExecutor() as executor:
+        try:
+            future_requests = [
+                executor.submit(
+                    _make_api_request,
+                    f"{base_url}/observation_json/get",
+                    access_token=access_token,
+                    params=query,
+                )
+                for query in queries
+            ]
+
+            for future_request in future_requests:
+                observation_data.extend(future_request.result())
+        except KeyboardInterrupt:
+            print("Cancelling requests")
+            cancelled_count = 0
+            total_count = 0
+            running_count = 0
+            for future_request in future_requests:
+                total_count += 1
+                if future_request.running():
+                    running_count += 1
+                if future_request.cancel():
+                    cancelled_count += 1
+            print(
+                f"Cancelled {cancelled_count}/{total_count} requests. "
+                f"Waiting for {running_count} running requests to complete..."
             )
-            for query in queries
-        ]
+            sys.exit()
 
-        for f in future_requests:
-            observation_data.extend(f.result())
-
-    if not len(observation_data):
+    if len(observation_data) == 0:
         return pd.DataFrame(
             columns=[
                 "from_dt",
@@ -155,7 +166,29 @@ def get_intraday_trades(
     return observation_data
 
 
-def get(
+def _split_series_ids(series_ids, access_token):
+    base_url = get_api_base_url_v2()
+    all_series_ids_observations = _make_api_request(
+        f"{base_url}/data/list_ids", access_token=access_token
+    )
+    all_series_ids_observations_json = _make_api_request(
+        f"{base_url}/observation_json/list_ids",
+        access_token=access_token,
+    )
+    series_ids_observations = [
+        series_id
+        for series_id in series_ids
+        if series_id in all_series_ids_observations["series_id"]
+    ]
+    series_ids_observations_json = [
+        series_id
+        for series_id in series_ids
+        if series_id in all_series_ids_observations_json["series_id"]
+    ]
+    return series_ids_observations, series_ids_observations_json
+
+
+def get(  # pylint:disable=too-many-locals,too-many-branches,too-many-statements
     series_ids: List[str],
     price_areas: List[str] = None,
     start: str = None,
@@ -186,9 +219,12 @@ def get(
 
     Args:
         series_ids (List[str]): The series ids to get.
-        price_areas (List[str], optional): The price areas to pick, all price areas if None. Defaults to None.
-        start (str, optional): The first datetime to fetch (inclusive). Returns all if None. Defaults to None.
-        end (str, optional): The last datetime to fetch (exclusive). Returns all if None. Defaults to None.
+        price_areas (List[str], optional): The price areas to pick, all price areas if
+                                           None. Defaults to None.
+        start (str, optional): The first datetime to fetch (inclusive). Returns all if
+                               None. Defaults to None.
+        end (str, optional): The last datetime to fetch (exclusive). Returns all if
+                             None. Defaults to None.
         access_token (str, optional): None - access token for the API
 
     Returns:
@@ -208,150 +244,157 @@ def get(
     if isinstance(price_areas, str):
         price_areas = [price_areas]
 
-    start = pd.to_datetime(start) if start else archimedes.ArchimedesConstants.DATE_LOW
-    end = pd.to_datetime(end) if end else archimedes.ArchimedesConstants.DATE_HIGH
+    start, end = get_start_date(start), get_end_date(end)
 
-    queries = [
-        {
-            "start": start,
-            "end": end,
-            "flatten_columns": True,
-        }
-    ]
-    if series_ids is not None and len(series_ids):
-        queries = [
-            {"series_ids": series_id, "series_id": series_id, **query}
-            for query in queries
-            for series_id in series_ids
-        ]
-    if price_areas is not None and len(price_areas):
-        queries = [
-            {"price_areas": price_area, **query}
-            for query in queries
-            for price_area in price_areas
-        ]
+    observation_series_ids, observation_json_series_ids = _split_series_ids(
+        series_ids, access_token
+    )
+
+    queries_by_type = {
+        "observations": get_queries_observation(
+            observation_series_ids, price_areas, start, end
+        ),
+        "observations_json": get_queries_observation_json(
+            observation_json_series_ids, price_areas, start, end
+        ),
+    }
 
-    df = None
     base_url = get_api_base_url_v2()
-    with concurrent.futures.ThreadPoolExecutor() as executor:
-        future_requests = {
-            "observations": [],
-            "observations_json": [],
-        }
+
+    def _make_observation_or_observation_json_request(request_type, query):
+        request_url_path = (
+            "data" if request_type == "observations" else "observation_json"
+        )
+        url = f"{base_url}/{request_url_path}/get"
+        query_result = _make_api_request(url, params=query, access_token=access_token)
+        return request_type, query_result
+
+    with ThreadPoolExecutor() as executor:
         try:
-            for query in queries:
-                future_requests["observations"].append(
-                    executor.submit(
-                        _make_api_request,
-                        f"{base_url}/data/get",
-                        access_token=access_token,
-                        params=query,
-                    )
-                )
-                future_requests["observations_json"].append(
-                    executor.submit(
-                        _make_api_request,
-                        f"{base_url}/observation_json/get",
-                        access_token=access_token,
-                        params=query,
-                    )
+            future_requests = []
+            for query_type, queries in queries_by_type.items():
+                future_requests.extend(
+                    [
+                        executor.submit(
+                            _make_observation_or_observation_json_request,
+                            query_type,
+                            query,
+                        )
+                        for query in queries
+                    ]
                 )
 
-            api_results = {
-                "observations": [f.result() for f in future_requests["observations"]],
-                "observations_json": [
-                    f.result() for f in future_requests["observations_json"]
-                ],
-            }
+            api_results = [
+                (future_request.result()) for future_request in future_requests
+            ]
         except KeyboardInterrupt:
             print("Cancelling requests")
             cancelled_count = 0
             total_count = 0
-            for _, fs in future_requests.items():
-                for f in fs:
-                    total_count += 1
-                    if f.cancel():
-                        cancelled_count += 1
+            running_count = 0
+            for future_request in future_requests:
+                total_count += 1
+                if future_request.running():
+                    running_count += 1
+                if future_request.cancel():
+                    cancelled_count += 1
             print(
                 f"Cancelled {cancelled_count}/{total_count} requests. "
-                f"Waiting for {total_count - cancelled_count} requests to complete..."
+                f"Waiting for {running_count} running requests to complete..."
             )
-            exit()
+            sys.exit()
 
-    for observation_type, observations_list in api_results.items():
-        for observations in observations_list:
-            if len(observations) == 0:
-                continue
+    observation_merged = []
+    observation_json_merged = []
 
-            observations = pd.DataFrame.from_dict(observations)
+    for (observation_type, observations) in api_results:
+        if len(observations) == 0:
+            continue
 
-            if observation_type == "observations":
-                observations = observations.sort_values(by=["from_dt", "version"])
-                observations = observations.pivot_table(
-                    values="value",
-                    columns=["series_id", "price_area"],
-                    index="from_dt",
-                    aggfunc="last",
-                )
-            elif observation_type == "observations_json":
-                observations[["series_id1", "series_id2", "price_area"]] = observations[
-                    "series_id"
-                ].str.split("/", 2, expand=True)
-                observations["series_id"] = (
-                    observations["series_id1"] + "/" + observations["series_id2"]
-                )
-                observations = observations.drop(["series_id1", "series_id2"], axis=1)
-                is_intraday_trades = observations["series_id"] == "NP/IntradayTrades"
+        if observation_type == "observations":
+            observation_merged.extend(observations)
+        elif observation_type == "observations_json":
+            observation_json_merged.extend(observations)
 
-                def agg_func(x):
-                    return list(x)
+    observations = pd.DataFrame(observation_merged)
+    if not observations.empty:
+        observations = observations.sort_values(by=["from_dt", "version"])
+        observations = observations.pivot_table(
+            values="value",
+            columns=["series_id", "price_area"],
+            index="from_dt",
+            aggfunc="last",
+        )
 
-                observation_data_intraday = observations[is_intraday_trades]
-                if len(observation_data_intraday):
-                    observation_data_intraday = observation_data_intraday.sort_values(
-                        by=["from_dt"]
-                    )
-                    observation_data_intraday = observation_data_intraday.pivot_table(
-                        values="value",
-                        columns=["series_id", "price_area"],
-                        index="from_dt",
-                        aggfunc=agg_func,
-                    )
-                else:
-                    observation_data_intraday = pd.DataFrame()
-                observations = observations[~is_intraday_trades]
-                if len(observations):
-                    observations = observations.sort_values(
-                        by=["from_dt", "version"]
-                    ).pivot_table(
-                        values="value",
-                        columns=["series_id", "price_area"],
-                        index="from_dt",
-                        aggfunc="last",
-                    )
-                else:
-                    observations = pd.DataFrame()
+    observations_json = pd.DataFrame(observation_json_merged)
+    if not observations_json.empty:
+        observations_json[
+            ["series_id1", "series_id2", "price_area"]
+        ] = observations_json["series_id"].str.split("/", 2, expand=True)
+        observations_json["series_id"] = (
+            observations_json["series_id1"] + "/" + observations_json["series_id2"]
+        )
+        observations_json = observations_json.drop(["series_id1", "series_id2"], axis=1)
+        is_intraday_trades = observations_json["series_id"] == "NP/IntradayTrades"
 
-                observations = observations.merge(
-                    observation_data_intraday,
-                    left_index=True,
-                    right_index=True,
-                    how="outer",
-                )
+        def agg_func(items):
+            return list(items)
 
-            df = (
-                observations
-                if df is None
-                else pd.merge(
-                    df,
-                    observations,
-                    left_index=True,
-                    right_index=True,
-                    how="outer",
-                )
+        observation_data_intraday = observations_json[is_intraday_trades]
+        if len(observation_data_intraday):
+            observation_data_intraday = observation_data_intraday.sort_values(
+                by=["from_dt"]
             )
+            observation_data_intraday = observation_data_intraday.pivot_table(
+                values="value",
+                columns=["series_id", "price_area"],
+                index="from_dt",
+                aggfunc=agg_func,
+            )
+
+        observations_json = observations_json[~is_intraday_trades]
+        if len(observations_json):
+            observations_json = observations_json.sort_values(
+                by=["from_dt", "version"]
+            ).pivot_table(
+                values="value",
+                columns=["series_id", "price_area"],
+                index="from_dt",
+                aggfunc="last",
+            )
+
+        if observation_data_intraday.empty and observations_json.empty:
+            observations_json = pd.DataFrame()
+        elif observation_data_intraday.empty:
+            pass
+        elif observations_json.empty:
+            observations_json = observation_data_intraday
+        else:
+            observations_json = observations_json.merge(
+                observation_data_intraday,
+                left_index=True,
+                right_index=True,
+                how="outer",
+            )
+
+    if observations_json.empty and observations.empty:
+        return pd.DataFrame()
+
+    if observations.empty:
+        df = observations_json
+    elif observations_json.empty:
+        df = observations
+    else:
+        df = pd.merge(
+            observations,
+            observations_json,
+            left_index=True,
+            right_index=True,
+            how="outer",
+        )
 
     df.index = pd.to_datetime(df.index)
+
     return df
 
 
@@ -386,7 +429,8 @@ def get_latest(
 
     Args:
         series_ids (List[str]): The series ids to get.
-        price_areas (List[str], optional): The price areas to pick, all price areas if None. Defaults to None.
+        price_areas (List[str], optional): The price areas to pick, all price areas if
+                                           None. Defaults to None.
         access_token (str, optional): None - access token for the API
 
     Returns:
@@ -412,6 +456,7 @@ def get_latest(
     return df
 
 
+# pylint:disable=too-many-arguments,too-many-locals,too-many-branches
 def get_predictions(
     series_ids: List[str] = None,
     price_areas: List[str] = None,
@@ -440,9 +485,11 @@ def get_predictions(
         series_ids (List[str], optional): The series ids to get.
         price_areas (List[str], optional): The price areas to get the data for.
         start (str, optional):
-            The first datetime to fetch (inclusive). Returns all if None. Defaults to None.
+            The first datetime to fetch (inclusive). Returns all if None.
+            Defaults to None.
         end (str, optional):
-            The last datetime to fetch (exclusive). Returns all if None. Defaults to None.
+            The last datetime to fetch (exclusive). Returns all if None.
+            Defaults to None.
         ref_dt_start (pd.Timestamp, optional):
             The earliest ref_dt to fetch (inclusive). Defaults to None.
         ref_dt_end (pd.Timestamp, optional):
@@ -489,7 +536,7 @@ def get_predictions(
 
     base_url = get_api_base_url_v2()
     data = []
-    with concurrent.futures.ThreadPoolExecutor() as executor:
+    with ThreadPoolExecutor() as executor:
         future_requests = [
             executor.submit(
                 _make_api_request,
@@ -500,8 +547,8 @@ def get_predictions(
             for query in queries
         ]
 
-        for f in future_requests:
-            for item in f.result():
+        for future_request in future_requests:
+            for item in future_request.result():
                 if "json_data" in item:
                     json_data = item.get("json_data")
                     item.update(json_data)
@@ -573,7 +620,8 @@ def list_ids(sort: bool = False, *, access_token: str = None):
         39                    NP/SpecialUpVolume
 
     Args:
-        sort (bool): False - return all series in one dataframe column, True - order dataframe by data-origin
+        sort (bool): False - return all series in one dataframe column, True - order
+                             dataframe by data-origin
         access_token (str, optional): None - access token for the API
 
     Returns:
@@ -632,7 +680,8 @@ def get_predictions_ref_dts(prediction_id: str = None, *, access_token: str = No
     Users views in the database.
 
     Args:
-        prediction_id (str): The series id to get the reference dts for. If None, get ref_dts for all prediction_ids.
+        prediction_id (str): The series id to get the reference dts for. If None, get
+                             ref_dts for all prediction_ids.
         access_token (str, optional): Access token for the API
 
     Returns:
@@ -653,12 +702,13 @@ def get_predictions_ref_dts(prediction_id: str = None, *, access_token: str = No
 
 
 @retry(RETRY_EXCEPTIONS, tries=3, delay=2)
-def _make_api_request(url, method="GET", access_token=None, *args, **kwargs):
+def _make_api_request(url, method="GET", access_token=None, **kwargs):
     if access_token is None:
         archimedes_auth = get_auth()
         if archimedes_auth is None:
             raise NoneAuth(
-                "access_token parameter must be passed when using USE_WEB_AUTHENTICATION"
+                "access_token parameter must be passed when using "
+                "USE_WEB_AUTHENTICATION"
             )
         access_token = archimedes_auth.get_access_token_silent()
 
@@ -672,12 +722,11 @@ def _make_api_request(url, method="GET", access_token=None, *args, **kwargs):
         headers = {}
 
     headers.update({"Authorization": f"Bearer {access_token}"})
+    response = requests.request(method, url, headers=headers, **kwargs)
 
-    r = requests.request(method, url, headers=headers, *args, **kwargs)
-
-    if r.status_code not in [HTTPStatus.OK, HTTPStatus.CREATED]:
+    if response.status_code not in [HTTPStatus.OK, HTTPStatus.CREATED]:
         try:
-            response_json = r.json()
+            response_json = response.json()
             if "message" in response_json:
                 error_message = response_json.get("message")
             elif "detail" in response_json:
@@ -685,10 +734,14 @@ def _make_api_request(url, method="GET", access_token=None, *args, **kwargs):
             else:
                 error_message = json.dumps(response_json)
         except JSONDecodeError:
-            error_message = r.content
-        raise HTTPError(f"API Error: {error_message}")
+            error_message = response.content
+        params_str = json.dumps(kwargs.get("params"))
+        raise HTTPError(
+            f"API Error while requesting {url} with parameters {params_str}: "
+            f"{error_message}"
+        )
 
-    return r.json()
+    return response.json()
 
 
 def store_prediction(
@@ -720,7 +773,7 @@ def store_prediction(
         >>>     data=data
         >>> )
         True
-    """
+    """  # pylint:disable=line-too-long
     payload = {
         "prediction_id": prediction_id,
         "from_dt": from_dt.isoformat(),
@@ -764,7 +817,7 @@ def store_predictions(
         >>>     prediction_data=prediction_data
         >>> )
         True
-    """
+    """  # pylint:disable=line-too-long
 
     predictions = [
         {
@@ -804,16 +857,20 @@ def forecast_list_ref_times(
     List all forecast reference times (the times that the forecast was generated)
 
     Args:
-        series_id (str): The ID of the data series to find all the forecast reference times (the time the forecast was
-            generated). Retrieve the complete list of series (both forecasts and observations) using the list_ids
-            resource.
-        start (pd.Timestamp, optional): The first datetime to fetch (inclusive). Returns all if not set. Should be
+        series_id (str):
+            The ID of the data series to find all the forecast reference times (the
+            time the forecast was generated). Retrieve the complete list of series
+            (both forecasts and observations) using the list_ids resource.
+        start (pd.Timestamp, optional):
+            The first datetime to fetch (inclusive). Returns all if not set. Should be
             specified in ISO 8601 format.
             (eg - '2021-11-29T06:00:00+00:00')
-        end (pd.Timestamp, optional): The last datetime to fetch (exclusive). Returns all if not set. Should be
+        end (pd.Timestamp, optional):
+            The last datetime to fetch (exclusive). Returns all if not set. Should be
             specified in ISO 8601 format.
             (eg - '2021-11-30T06:00:00+00:00')
-        limit (int, optional): Limit the output to a specific number of entries. No limit if not specified.
+        limit (int, optional):
+            Limit the output to a specific number of entries. No limit if not specified.
         access_token (str, optional): Access token for the API
 
     Example:
@@ -829,7 +886,7 @@ def forecast_list_ref_times(
 
     Returns:
         Dataframe with all of the forecast reference times
-    """
+    """  # pylint:disable=line-too-long
     query = {
         "series_id": series_id,
         "start": start.isoformat() if start is not None else None,
@@ -859,33 +916,42 @@ def forecast_diff(
     Get the difference between two different forecasts.
 
     Args:
-        comparison_type: The type of comparison to do to the two forecasts:
-            forecast_update - how has the the forecast for a specific time range been updated. The two forecast
-                reference times must be within ~60 hours of each other. Otherwise the output will be empty (because
-                the forecasts don't overlap).
-
-            forecast_diff - compares the forecasts of any two dates to indicate how different they are.
-        series_ids: The ID of the data series to get (eg - 'MET/forecast_air_temperature_2m' or
-            'MET/forecast_wind_speed_10m'). To specify multiple data series, include the series_ids parameter multiple
-            times in the url.
+        comparison_type:
+            The type of comparison to do to the two forecasts:
+                forecast_update - how has the forecast for a specific time range been
+                    updated. The two forecast reference times must be within ~60 hours
+                    of each other. Otherwise, the output will be empty (because
+                    the forecasts don't overlap).
+                forecast_diff - compares the forecasts of any two dates to indicate how
+                    different they are.
+        series_ids:
+            The ID of the data series to get (eg - 'MET/forecast_air_temperature_2m' or
+            'MET/forecast_wind_speed_10m'). To specify multiple data series, include
+            the series_ids parameter multiple times in the url.
             Retrieve the complete list of series using the list_ids resource.
-        price_areas: The name of the price area(eg - 'NO2', 'NO5', or 'DE1-NO1'). To specify multiple price areas,
-            include the price_areas parameter multiple times in the url.
-            Retrieve the complete list of price areas available for a specified series ID using list_series_price_areas
-            resource.
-        ref_time1: Specify one of the two timestamps for when a forecast was created. Should be specified in ISO 8601
-            format (eg - '2021-11-29T06:00:00+00:00').
-        ref_time2: Specify another of the two timestamps for when a forecast was created. Should be specified in ISO
-            8601 format (eg - '2021-11-29T06:00:00+00:00').
-        access_token (str, optional): Access token for the API
+        price_areas:
+            The name of the price area(eg - 'NO2', 'NO5', or 'DE1-NO1'). To specify
+            multiple price areas, include the price_areas parameter multiple times in
+            the url.
+            Retrieve the complete list of price areas available for a specified series
+            ID using list_series_price_areas resource.
+        ref_time1:
+            Specify one of the two timestamps for when a forecast was created. Should
+            be specified in ISO 8601 format (eg - '2021-11-29T06:00:00+00:00').
+        ref_time2:
+            Specify another of the two timestamps for when a forecast was
+            created. Should be specified in ISO 8601 format
+            (eg - '2021-11-29T06:00:00+00:00').
+        access_token (str, optional):
+            Access token for the API
 
     Returns:
         Dataframe with the diff of the two forecasts
     """
-    assert comparison_type in [
-        "forecast_update",
-        "forecast_diff",
-    ], f"Unknown comparison_type '{comparison_type}' (should be either 'forecast_update' or 'forecast_diff')"
+    assert comparison_type in ["forecast_update", "forecast_diff",], (
+        f"Unknown comparison_type '{comparison_type}' "
+        f"(should be either 'forecast_update' or 'forecast_diff')"
+    )
     query = {
         "forecast_comparison_type": comparison_type,
         "series_ids": series_ids,
@@ -934,9 +1000,12 @@ def forecast_get(
 
     Args:
         series_ids (List[str]): The series ids to get.
-        price_areas (List[str], optional): The price areas to pick, all price areas if None. Defaults to None.
-        start (str, optional): The first datetime to fetch (inclusive). Returns all if None. Defaults to None.
-        end (str, optional): The last datetime to fetch (exclusive). Returns all if None. Defaults to None.
+        price_areas (List[str], optional): The price areas to pick, all price areas if
+                                           None. Defaults to None.
+        start (str, optional): The first datetime to fetch (inclusive). Returns all if
+                               None. Defaults to None.
+        end (str, optional): The last datetime to fetch (exclusive). Returns all if
+                             None. Defaults to None.
         access_token (str, optional): None - access token for the API
 
     Returns:
@@ -953,14 +1022,12 @@ def forecast_get(
     if isinstance(price_areas, str):
         price_areas = [price_areas]
 
-    start = pd.to_datetime(start) if start else archimedes.ArchimedesConstants.DATE_LOW
-    end = pd.to_datetime(end) if end else archimedes.ArchimedesConstants.DATE_HIGH
+    start, end = get_start_date(start), get_end_date(end)
 
     queries = [
         {
             "start": start,
             "end": end,
-            "flatten_columns": True,
         }
     ]
     queries = [
@@ -976,7 +1043,7 @@ def forecast_get(
 
     base_url = get_api_base_url_v2()
     data = []
-    with concurrent.futures.ThreadPoolExecutor() as executor:
+    with ThreadPoolExecutor() as executor:
         future_requests = [
             executor.submit(
                 _make_api_request,
@@ -987,16 +1054,16 @@ def forecast_get(
             for query in queries
         ]
 
-        for f in future_requests:
-            data.extend(f.result())
+        for future_request in future_requests:
+            data.extend(future_request.result())
 
-    if len(data):
-        df = pd.DataFrame.from_dict(data)
-        df = df.sort_values(by=["from_dt", "ref_dt"])
-    else:
+    if len(data) == 0:
         df = pd.DataFrame(
             columns=["series_id", "from_dt", "ref_dt", "value", "price_area"]
         )
+    else:
+        df = pd.DataFrame.from_dict(data)
+        df = df.sort_values(by=["from_dt", "ref_dt"])
 
     df["from_dt"] = pd.to_datetime(df["from_dt"])
     df["ref_dt"] = pd.to_datetime(df["ref_dt"])
@@ -1020,25 +1087,35 @@ def forecast_get_by_ref_time_interval(
     access_token: str = None,
 ):
     """
-    Get a single forecast value for every hour. The value should be from the forecast that was generated at least
-    forecast_interval hours prior.
+    Get a single forecast value for every hour. The value should be from the forecast
+    that was generated at least forecast_interval hours prior.
 
     Args:
-        series_id: The ID of the data series to get (eg - 'MET/forecast_air_temperature_2m' or
-            'MET/forecast_wind_speed_10m'). Retrieve the complete list of series using the list_ids resource.
-        price_area: The name of the price area(eg - 'NO2', 'SE3'). Retrieve the complete list of price areas available
-            for a specified series ID using list_series_price_areas resource.
-        start: The first datetime to fetch (inclusive). Returns all if not set. Should be specified in ISO 8601 format
-            (eg - '2021-11-29T06:00:00+00:00')
-        end: The last datetime to fetch (exclusive). Returns all if not set. Should be specified in ISO 8601 format
-            (eg - '2021-11-30T06:00:00+00:00')
-        forecast_interval: The number of hours earlier that the forecast must have been generated. In some cases, it
-            could be older (if no forecast was generated at exactly that hour). NOTE - this is ignored if
-            day_ahead_hour is set.
-        day_ahead_hour: Used for day-ahead market. Indicates the hour of the day when the market closes (CET - Central
-            European Time). Will return the forecast generated before this time on the previous day
-            ('forecast_interval' will be set to 24). For example, if set to '12' (noon CET), the values shown for every
-            hour of a specific day will be fetched from the most recent forecast generated before noon (most likely
+        series_id:
+            The ID of the data series to get (eg - 'MET/forecast_air_temperature_2m' or
+            'MET/forecast_wind_speed_10m'). Retrieve the complete list of series using
+            the list_ids resource.
+        price_area:
+            The name of the price area(eg - 'NO2', 'SE3'). Retrieve the complete list
+            of price areas available for a specified series ID using
+            list_series_price_areas resource.
+        start:
+            The first datetime to fetch (inclusive). Returns all if not set. Should be
+            specified in ISO 8601 format (eg - '2021-11-29T06:00:00+00:00')
+        end:
+            The last datetime to fetch (exclusive). Returns all if not set. Should be
+            specified in ISO 8601 format (eg - '2021-11-30T06:00:00+00:00')
+        forecast_interval:
+            The number of hours earlier that the forecast must have been generated. In
+            some cases, it could be older (if no forecast was generated at exactly that
+            hour). NOTE - this is ignored if day_ahead_hour is set.
+        day_ahead_hour:
+            Used for day-ahead market. Indicates the hour of the day when the market
+            closes (CET - Central European Time). Will return the forecast generated
+            before this time on the previous day
+            ('forecast_interval' will be set to 24). For example, if set to '12'
+            (noon CET), the values shown for every hour of a specific day will be
+            fetched from the most recent forecast generated before noon (most likely
             11am) on the previous day.
         access_token (str, optional): Access token for the API
 
